<?php

namespace Intergo\LaravelAuthMiddleware;

use Illuminate\Contracts\Container\BindingResolutionException;
use Illuminate\Contracts\Http\Kernel;
use Illuminate\Support\ServiceProvider;
use Intergo\LaravelAuthMiddleware\Http\Middleware\Authenticate;
use Intergo\Log\Contracts\LoggingFieldsEnum;
use Intergo\Log\GrayLogHandler;
use Psr\Log\LoggerInterface;
use Str;

class LaravelAuthMiddlewareServiceProvider extends ServiceProvider
{
    /**
     * Register services.
     */
    public function register(): void
    {
        $this->app->singleton(GrayLogHandler::class, function ($app) {
            $trace_id = $app->request->header('X-TRACE-ID');
            if(empty($trace_id)){
                $trace_id = Str::random(32);
            }
            $gray_logger_handler = new GrayLogHandler($app->make(LoggerInterface::class), LoggingFieldsEnum::class);
            $gray_logger_handler->setTraceId($trace_id);
            return $gray_logger_handler;
        });
    }

    /**
     * Bootstrap services.
     * @throws BindingResolutionException
     */
    public function boot(): void
    {
        $this->publishes([
            __DIR__.'/../config/auth-middleware.php' => config_path('auth-middleware.php'),
        ]);

        $this->configureMiddleware();
    }

    /**
     * @throws BindingResolutionException
     */
    protected function configureMiddleware(): void
    {
        $kernel = app()->make(Kernel::class);

        $kernel->prependToMiddlewarePriority(Authenticate::class);
    }
}
